@@ -14,9 +14,6 @@ import pty
 import struct
 import fcntl
 import signal
-import termios
-import struct
-import fcntl
 import logging
 import argparse
 import importlib
@@ -29,7 +26,7 @@ from subprocess import Popen
 
 from .config import load_config
 from . import exceptions
-from .utils import load_server_db
+from .utils import load_server_db, read_user_input_line
 from .server_killer import killer
 from . import __version__
 
@@ -189,7 +186,11 @@ class Server(paramiko.ServerInterface):
         self, channel, width, height, pixelwidth, pixelheight
     ):
         logger.debug(
-            "client send window size change reuqest... width={}, height={}, pixelwidth={}, pixelheight={}, my proxy_subprocess_pid={}, master_fd={}".format(
+            (
+                "client send window size change reuqest... "
+                "width={}, height={}, pixelwidth={}, pixelheight={}, "
+                "my proxy_subprocess_pid={}, master_fd={}"
+            ).format(
                 width,
                 height,
                 pixelwidth,
@@ -244,7 +245,7 @@ class SocketHandlerThread(threading.Thread):
             t.set_gss_host(socket.getfqdn(""))
             try:
                 t.load_server_moduli()
-            except:
+            except:  # noqa
                 logger.error("(Failed to load moduli -- gex will be unsupported.)")
                 raise
             host_key = paramiko.RSAKey(
@@ -307,7 +308,7 @@ class SocketHandlerThread(threading.Thread):
             chan.send((int(server.window_width) * "=" + "\r\n").encode())
             chan_fd = chan.fileno()
             while p.poll() is None:
-                r, w, e = select.select([master_fd, chan_fd], [], [], 0.1)
+                r, _, _ = select.select([master_fd, chan_fd], [], [], 0.1)
                 if master_fd in r:
                     d = os.read(master_fd, 10240)
                     chan.send(d)
@@ -325,13 +326,15 @@ class SocketHandlerThread(threading.Thread):
             chan.shutdown(0)
             t.close()
 
-        except Exception as e:
-            logger.error("*** Caught exception: " + str(e.__class__) + ": " + str(e))
-            traceback.print_exc()
+        except Exception:
+            logger.critical(
+                "*** Socket thread error.",
+                exc_info=True,
+            )
             try:
                 evict_active_session(t, serverid)
                 t.close()
-            except:
+            except:  # noqa
                 pass
 
     def load_server_info(self, serverid):
@@ -391,7 +394,7 @@ class SocketHandlerThread(threading.Thread):
                     )
                 )
             channel.send("Please input your choice (number): ".encode())
-            user_input = int(self.read_user_input_line(channel))
+            user_input = int(read_user_input_line(channel))
             if user_input == 0:
                 logger.info("userinput=0, wants to create a new server...")
                 serverid, serverhost, provider = self._create_new_server(channel)
@@ -444,7 +447,7 @@ class SocketHandlerThread(threading.Thread):
             for index, name in enumerate(pnames):
                 channel.send("{:>3} - {}\r\n".format(index, name))
             channel.send("Please choose a provider to create a new server: ")
-            user_input = self.read_user_input_line(channel)
+            user_input = read_user_input_line(channel)
             choosed_provider_name = pnames[int(user_input)]
             return self.providers[choosed_provider_name]
 
@@ -471,22 +474,6 @@ class SocketHandlerThread(threading.Thread):
 
         return server_id, server_ip, provider
 
-    def read_user_input_line(self, chan):
-        # TODO do not support del
-        logger.debug("reading from channel... {}".format(chan))
-        chars = []
-        while 1:
-            content = chan.recv(1)
-            logger.debug("channel recv: {}".format(content))
-            if content == b"\r":
-                chan.send(b"\r\n")
-                break
-            if content == b"\x04" or content == b"\x03":
-                raise exceptions.UserCancelException()
-            chan.send(content)
-            chars.append(content)
-        return b"".join(chars).decode()
-
 
 def runserver(config, providers):
     try:
@@ -570,7 +557,7 @@ def main():
     killer_thread = threading.Thread(
         target=killer,
         args=(config, active_session, available_server_db_lock, providers),
-        daemon=True
+        daemon=True,
     )
     killer_thread.start()
     logger.info("started server_killer thread: {}".format(killer_thread))
